// indexRedirect is injected into web frontend's index.html to redirect the
// browser to the webapp oauth2 client to log in the user (if needed)
function indexRedirect() {
    // console.debug("cui", cui)
    let url = new URL(document.location.href)
    let up = new URLSearchParams(document.location.search)
    let server = up.get("server")
    let cui = loadCUISession()

    // token is from a cross server login
    if (up.get("access_token")) {
        cui.token = up.get("access_token")
        cui.refreshToken = up.get("refresh_token")
        cui.tokenIsFresh = true
    }

    console.debug("redirect.js: url", url, "params", up, "token", cui.token, "refresh", cui.refreshToken, "fresh", cui.tokenIsFresh)
    if (up.get("server") && !cui.tokenIsFresh) {
        // remove cui session
        cui = {}
        storeCUISession(cui)
    }
    if (cui.tokenIsFresh) {
        // remove the tokenIsFresh
        delete(cui.tokenIsFresh)
        storeCUISession(cui)
    }

    // check all possible deep links which require an OAUTH2
    // handling of the connection
    let upR = new URLSearchParams()
    // console.debug("upR", upR)
    let match = document.location.pathname.match("^/collection/(.*)$")
    if (match) {
        // collection
        if (url.searchParams.get("collection")) {
            upR.set("auth_method", "collection")
            upR.set("login", url.searchParams.get("collection")) // this is different from ez5 where login & password are swapped!
            upR.set("password", match[1])
            url.searchParams.delete("collection")
        } else if (url.searchParams.has("anonymous")) {
            // anonmyous
            upR.set("auth_method", "anonymous")
        } else if (url.searchParams.get("email") && url.searchParams.get("uuid")) {
            upR.set("auth_method", "email")
            upR.set("login", url.searchParams.get("email"))
            upR.set("password", url.searchParams.get("uuid"))
            url.searchParams.delete("email")
            url.searchParams.delete("uuid")
        }
    }

    if (upR.get("auth_method")) {
        // This overrides all token
        delete(cui.token)
        storeCUISession(cui)
    }

    if (cui.token) {
        // the CUI session storage contains a token. This might have
        // been set by "loginRedirect" or set already (in case of a page reload)
        // we continue to load the web frontend with no redirect
        console.debug("have token, continue")
    } else {
        server = up.get("server") || cui.server
        if (server && server != "localhost" && server != window.fylr_server) {
            // let the request pass, this is cross server
            console.debug("url or session contains server, letting this pass for cross server web frontend",  "url", up.get("server"), "cui.server", cui.server)
        } else {
            // redirect to login (oauth webapp client), unless webOnly mode
            if (window.isWebOnly) {
                console.debug("no token, webOnly mode, redirect to /webOnly")
                document.location.assign(window.easydb_base_prefix+"/webOnly")
            } else {
                // set auth method to auto if not requested explicitly
                if (!upR.get("auth_method")) {
                    upR.set("auth_method", "auto")
                }

                // pass user provided login + password from url, remove
                // it from the redirect
                if (up.get("login")) {
                    url.searchParams.delete("login")
                    url.searchParams.delete("password")
                    upR.set("login", up.get("login"))
                    upR.set("password", up.get("password"))
                }

                upR.set("redirect", url.toString())

                let rURL = window.easydb_base_prefix+"/login?"+upR.toString()
                console.debug("no token, redirect to oauth", rURL)
                document.location.assign(rURL)
            }
        }
    }
}

// loginRedirect is used by the oauth2 callback to instruct the browser to
// redirect to the web frontend. this sets the token in sessionStorage ready for
// the web frontend to use.
function loginRedirect(fylr_access_token, fylr_refresh_token, rURL) {
    let redirect = undefined

    try {
        redirect = new URL(rURL)
    } catch (e) {
        // url not valid
        document.write("<p>Invalid Redirect URL '"+rURL+"'</p>")
        return
    }

    if (redirect.origin === document.location.origin) {
        // the redirect is on the same location
        // we can use the session storage to transport the session
        let cui = loadCUISession()
        cui.token = fylr_access_token
        cui.refreshToken = fylr_refresh_token
        cui.tokenIsFresh = true
        storeCUISession(cui)
        console.debug("redirect with access & refresh token in sessionStorage", redirect)
    } else {
        // this is a cross server redirect, we must pass
        // the token in the URL
        redirect.searchParams.set("access_token", fylr_access_token)
        redirect.searchParams.set("refresh_token", fylr_refresh_token)
        console.debug("redirect with access & refresh token in URL (cross server)", redirect)
    }
    document.location.assign(redirect.toString())
}

function storeCUISession(cui) {
    window.sessionStorage.setItem("CUI", JSON.stringify(cui))
}

// loadCUISession loads the session storage of CUI (as done by coffeescript-ui)
function loadCUISession() {
    let cui = window.sessionStorage.getItem("CUI");
    if (cui) {
        try {
            cui = JSON.parse(cui);
        } catch {
            // ignore
            cui = {}
        }
    } else {
        cui = {}
    }
    return cui
}

//fetch('http://localhost:8070/oauth2/refresh', {method: 'POST', body: JSON.stringify({access_token: CUI.getSessionStorage("token")}), headers: {'Content-Type': 'application/json'}})